"""
The CFD solvers
===============

Provides source code of 2D CFD solver.
The content is limited for the moment to Finite Differences.

Differenciation is done in *fd_operators*.
The actual Navier-Stokes solver is in *fd_ns_2d*.

"""
from .fd_ns_2d import *
from .fd_operators import *
from .fluid_domain import *



"""Module to handle Finite diff operators"""

import numpy as np

__all__ = ["grad_1", "grad_2", "scnd_der_2"]

#pylint: disable=unsupported-assignment-operation,invalid-sequence-index

def grad_1st_order(array_, d_x, perio=True):
    """Compute gradient of array (2nd order centered)"""
    grad_ = np.ones_like(array_)
    grad_[:, 1:-1] = (array_[:, 1:-1] - array_[:, 0:-2])/d_x
    if perio:
        grad_[:, -1] = (array_[:, -1] - array_[:, -2])/d_x
        grad_[:, 0] = (array_[:, 0] - array_[:, -2])/d_x
    else:
        grad_[:, -1] = (array_[:, -1] - array_[:, -2])/d_x
        grad_[:, 0] = (array_[:, 1] - array_[:, 0])/d_x
    return grad_


def grad_2nd_order(array_, d_x, perio=True):
    """Compute gradient of array (2nd order centered)"""
    grad_ = np.zeros_like(array_)
    grad_[:, 1:-1] = (array_[:, 2:] - array_[:, 0:-2])/(2*d_x)
    if perio:
        grad_[:, -1] = (array_[:, 1] - array_[:, -2])/(2*d_x)
        grad_[:, 0] = (array_[:, 1] - array_[:, -2])/(2*d_x)
    else:
        grad_[:, -1] = (array_[:, -1] - array_[:, -2])/d_x
        grad_[:, 0] = (array_[:, 1] - array_[:, 0])/d_x
    return grad_


def gradgrad_2nd_order(array_, d_x, perio=True, no_center=False):
    """Compute gradient of array (2nd order centered)"""
    ggrad = np.zeros_like(array_)
    if no_center:
        coef_center = 0
    else:
        coef_center = 2
    ggrad[:, 1:-1] = (
        array_[:, 2:]
        - coef_center*array_[:, 1:-1]
        + array_[:, 0:-2])/d_x**2

    if perio:
        ggrad[:, -1] = (array_[:, 1]
                        - coef_center*array_[:, -1]
                        + array_[:, -2])/d_x**2
        ggrad[:, 0] = (array_[:, 1]
                       - coef_center*array_[:, 0]
                       + array_[:, -2])/d_x**2
    else:
        ggrad[:, -1] = 2*ggrad[:, -2] - ggrad[:, -3]
        ggrad[:, 0] = 2*ggrad[:, 1] - ggrad[:, 2]
    return ggrad


def grad_1(array_, d_x, periox=False, perioy=False):
    """Compute the two gradients of an array."""
    g_x = grad_1st_order(array_, d_x, perio=periox)
    g_y = grad_1st_order(
        np.swapaxes(array_, 0, 1),
        d_x,
        perio=perioy)
    g_y = np.swapaxes(g_y, 0, 1)
    return g_x, g_y


def grad_2(array_, d_x, periox=False, perioy=False):
    """Compute the two gradients of an array."""
    g_x = grad_2nd_order(array_, d_x, perio=periox)
    g_y = grad_2nd_order(
        np.swapaxes(array_, 0, 1),
        d_x,
        perio=perioy)
    g_y = np.swapaxes(g_y, 0, 1)
    return g_x, g_y


def scnd_der_2(array_, d_x, periox=False, perioy=False, no_center=False):
    """Compute the two scnd derivatives of an array."""
    gg_x = gradgrad_2nd_order(
        array_,
        d_x,
        perio=periox,
        no_center=no_center)
    gg_y = gradgrad_2nd_order(
        np.swapaxes(array_, 0, 1),
        d_x,
        perio=perioy,
        no_center=no_center)
    gg_y = np.swapaxes(gg_y, 0, 1)
    return gg_x, gg_y



"""Example on how to solve a pollution problem with the navier stokes solver"""

import numpy as np

from barbatruc.fluid_domain import DomainRectFluid
from barbatruc.fd_ns_2d import ns_iteration

__all__ = ["pollution_ns"]


def pollution_ns(nsave, t_end):
    """Startup computation
    solve a pollution problem with the navier stokes solver
    """
    dom = DomainRectFluid(dimx=0.82, dimy=0.61, delta_x=0.02)
    dom.fields["vel_u"] = 1. * np.ones(dom.shape)
    dom.fields["vel_v"] = 0. * np.ones(dom.shape)
    dom.add_obstacle_circle()
    dom.bcx_inlet_uv_outlet_p(val_u=1., val_v=0.0)
    dom.bcy_noslip_channel()

    time = 0.0
    time_step = t_end/nsave

    ns_fields = None
    for i in range(nsave):
        time += time_step
        ns_fields = ns_iteration(dom, time_step, ns_fields)
        print('  Max u:', np.max(dom.fields["vel_u"]))
        print('  Time :', time)
        print('  Iteration %d' % (i))

    dom.show_fields()
    dom.show_flow()

    print('Normal end of execution.')


if __name__ == "__main__":
    pollution_ns(10, 0.40)



"""Module to solve 2D Naviers stokes equations.

The interactive module 12 steps to Navier-Stokes is one of several
components of the Computational Fluid Dynamics class taught by Professor
Lorena Barba in Boston university between 2009 and 2013.

The following code is started from step12
See https://nbviewer.jupyter.org/github/barbagroup/CFDPython/tree/master/lessons/
for futher reading

Dom here after isthe FluidDomain object of barbatruc
"""

import numpy as np
from barbatruc.fd_operators import (grad_2, scnd_der_2)



CFL_MAX = 0.699
FOU_MAX = 0.099
OBS_GAIN = -20.

__all__ = ["ns_iteration"]


def mini_ns_delta_t(dom):
    """ Define the mai iteration of the temporal loop.
    """
    u_max = max(np.max(np.hypot(
        dom.fields["vel_u"],
        dom.fields["vel_v"])), 1.e-8)
    delta_t_cfl = CFL_MAX*dom.delta_x/u_max
    delta_t_fou = FOU_MAX/dom.nu_*dom.delta_x**2
    delta_t = min(delta_t_cfl, delta_t_fou)
    return delta_t


def ns_iteration(
        dom,
        time_step,
        ns_fields=None, obs_immersed=0.0):
    """Major iteration."""
    time = 0.
    if ns_fields is None:
        ns_fields = dict()
        ns_fields["obs_force_x"] = np.zeros(dom.shape)
        ns_fields["obs_force_y"] = np.zeros(dom.shape)
        ns_fields["temporal"] = dict()
        for key in dom.fields:
            ns_fields["temporal"][key] = list()

    while time < time_step:
        delta_t = min(mini_ns_delta_t(dom), time_step-time)
        time += delta_t
        dom.fields = ns_sub_iteration(dom, ns_fields, delta_t, obs_immersed)
        for key in dom.fields:
            ns_fields["temporal"][key].append(dom.fields[key].mean())

    return ns_fields


def ns_sub_iteration(dom, ns_fields, delta_t, obs_immersed):
    """Sub iteration of Naviers stokes."""

    periox, perioy = dom.perio_xy()

    fields_new = dict()
    grad_x, grad_y, ggrad_x, ggrad_y = compute_derivatives(dom, periox, perioy)

    # compute pressure term
    fields_new["press"] = pressure_poisson(
        dom,
        delta_t,
        periox,
        perioy)

    if obs_immersed > 0.0:
        ns_fields["obs_force_x"] += obs_immersed*dom.obstacle*dom.fields["vel_u"]
        ns_fields["obs_force_y"] += obs_immersed*dom.obstacle*dom.fields["vel_v"]

    sce_term = dict()
    sce_term["vel_u"] = (
        dom.source["vel_u"]
        - grad_x["press"] / (2. * dom.rho)
        + ns_fields["obs_force_x"])
    sce_term["vel_v"] = (
        dom.source["vel_v"]
        - grad_y["press"] / (2. * dom.rho)
        + ns_fields["obs_force_y"])
    sce_term["scal"] = dom.source["scal"]

    for field in ["vel_u", "vel_v", "scal"]:
        fields_new[field] = dom.fields[field] + delta_t * (
            - (dom.fields["vel_u"] * grad_x[field] + dom.fields["vel_v"] * grad_y[field])
            + sce_term[field]
            + dom.nu_ * (ggrad_x[field] + ggrad_y[field])
            )

    apply_bc_dirichlet(dom, fields_new)

    if obs_immersed == 0.0:
        for field in ["vel_u", "vel_v"]:
            fields_new[field] = dom.nullify_on_obstacle(fields_new[field])

    return fields_new


def compute_derivatives(dom, periox, perioy):
    """Compute the derivatives of fields."""
    grad_x = dict()
    grad_y = dict()
    ggrad_x = dict()
    ggrad_y = dict()
    for field in ["vel_u", "vel_v", "scal", "press"]:
        grad_x[field], grad_y[field] = grad_2(
            dom.fields[field],
            dom.delta_x,
            periox=periox,
            perioy=perioy
            )
        ggrad_x[field], ggrad_y[field] = scnd_der_2(
            dom.fields[field],
            dom.delta_x,
            periox=periox,
            perioy=perioy)

    apply_bc_neumann(dom, grad_x, grad_y)
    return grad_x, grad_y, ggrad_x, ggrad_y


def apply_bc_dirichlet(dom, fields):
    """Apply boundary condituons with a crude manner."""
    if dom.bc_xmin_type == "dirichlet":
        for key in dom.bc_xmin_values.keys():
            fields[key][:, 0] = dom.bc_xmin_values[key]
    if dom.bc_xmax_type == "dirichlet":
        for key in dom.bc_xmax_values.keys():
            fields[key][:, -1] = dom.bc_xmax_values[key]
    if dom.bc_ymin_type == "dirichlet":
        for key in dom.bc_ymin_values.keys():
            fields[key][0, :] = dom.bc_ymin_values[key]
    if dom.bc_ymax_type == "dirichlet":
        for key in dom.bc_ymax_values.keys():
            fields[key][-1, :] = dom.bc_ymax_values[key]


def apply_bc_neumann(dom, derivx, derivy):
    """Apply boundary condituons with a crude manner."""
    if dom.bc_xmin_type == "neumann":
        for key in dom.bc_xmin_values.keys():
            derivx[key][:, 0] = dom.bc_xmin_values[key]
    if dom.bc_xmax_type == "neumann":
        for key in dom.bc_xmax_values.keys():
            derivx[key][:, -1] = dom.bc_xmax_values[key]
    if dom.bc_ymin_type == "neumann":
        for key in dom.bc_ymin_values.keys():
            derivy[key][0, :] = dom.bc_ymin_values[key]
    if dom.bc_ymax_type == "neumann":
        for key in dom.bc_ymax_values.keys():
            derivy[key][-1, :] = dom.bc_ymax_values[key]


def pressure_poisson(dom, delta_t, periox, perioy):
    """Define Pressure Poisson iterative ffields["vel_u"]_oldction.

    here ggrad_p_x and ggrad_p_y are terms comming from the pressure laplacian
    u_term is the pressure term coming from div_u
    """
    b_press = press_from_div_u(dom, delta_t, periox, perioy)
    eps_old = 1.
    press = dom.fields["press"].copy()
    norm_ref = np.linalg.norm(press)
    for _ in range(40):
        p_old = press.copy()
        ggrad_p_x, ggrad_p_y = scnd_der_2(
            p_old,
            dom.delta_x,
            periox=periox,
            perioy=perioy,
            no_center=True)
        press = (ggrad_p_x + ggrad_p_y - b_press) * dom.delta_x**2 / 4.0
        eps = np.linalg.norm(press-p_old)/norm_ref
        if eps < 1e-3:
            break
        if eps_old > eps:
            break
        eps_old = eps
    press -= press.mean()
    return press


def press_from_div_u(dom, delta_t, periox, perioy):
    """Compute the presure term from velocity divengence."""
    b_press = np.zeros_like(dom.fields["vel_u"])
    gux, guy = grad_2(
        dom.fields["vel_u"],
        dom.delta_x,
        periox=periox,
        perioy=perioy)
    gvx, gvy = grad_2(
        dom.fields["vel_v"],
        dom.delta_x,
        periox=periox,
        perioy=perioy)
    b_press = dom.rho * ((gux + gvy)/delta_t - (gux**2 + 2.0*guy*gvx + gvy**2))
    return b_press



"""Module to define a fluid domain and the configurations associated"""

import numpy as np
import matplotlib.pyplot as plt


__all__ = ["DomainRectFluid", "blur"]

#pylint: disable=too-many-instance-attributes
class DomainRectFluid():
    """docstring for Domain"""

    #pylint: disable=too-many-arguments
    def __init__(
            self,
            dimx=0.82,
            dimy=0.6,
            delta_x=0.02,
            rho=1.12,
            nu_=1.0):
        """Startup class"""
        n_nodes_x = int(dimx/delta_x)
        n_nodes_y = int(dimy/delta_x)
        self.dims = (n_nodes_y*delta_x, n_nodes_x*delta_x)
        self.shape = (n_nodes_y, n_nodes_x)
        self.delta_x = delta_x

        self.fields = dict()
        self.source = dict()
        for field in ["vel_u", "vel_v", "scal"]:
            self.fields[field] = np.zeros(self.shape)
            self.source[field] = np.zeros(self.shape)

        self.fields["press"] = np.ones(self.shape)*1.0
        self.obstacle = np.zeros(self.shape)

        self.rho = rho
        self.nu_ = nu_

        self.bc_xmax_type = "Perio"
        self.bc_xmax_values = dict()
        self.bc_xmin_type = "Perio"
        self.bc_xmin_values = dict()
        self.bc_ymax_type = "Perio"
        self.bc_ymax_values = dict()
        self.bc_ymin_type = "Perio"
        self.bc_ymin_values = dict()

    def bcx_inlet_uv_outlet_p(self, val_u=20.0, val_v=0.0):
        """ set to a dirichlet vel_u, vel_v."""
        self.bc_xmin_type = "dirichlet"
        self.bc_xmin_values = {"vel_u": val_u, "vel_v": val_v, "scal": 0.}

    def bcx_perio(self):
        """ set to a x-periodic domain."""
        self.bc_xmax_type = "Perio"
        self.bc_xmax_values = dict()
        self.bc_xmin_type = "Perio"
        self.bc_xmin_values = dict()

    def bcx_closed(self):
        """ set to a x no slip wall-wall domain."""
        self.bc_xmax_type = "dirichlet"
        self.bc_xmax_values = {"vel_u": 0., "vel_v": 0.}
        self.bc_xmin_type = "dirichlet"
        self.bc_xmin_values = {"vel_u": 0., "vel_v": 0.}

    def bcy_perio(self):
        """ set to a y-periodic domain."""
        self.bc_ymax_type = "Perio"
        self.bc_ymax_values = dict()
        self.bc_ymin_type = "Perio"
        self.bc_ymin_values = dict()

    def bcy_lid_driven(self, vel_u=20.):
        """Set to lowaer y wall, and a upper y moving wall."""
        self.bc_ymax_type = "dirichlet"
        self.bc_ymax_values = {"vel_u": vel_u, "vel_v": 0.}
        self.bc_ymin_type = "dirichlet"
        self.bc_ymin_values = {"vel_u": 0., "vel_v": 0.}

    def bcy_noslip_channel(self):
        """Set to a dy noslip wall-wall."""
        self.bc_ymax_type = "dirichlet"
        self.bc_ymax_values = {"vel_u": 0., "vel_v": 0.}
        self.bc_ymin_type = "dirichlet"
        self.bc_ymin_values = {"vel_u": 0., "vel_v": 0.}

    def bcy_slip_channel(self):
        """Set to a dy slip wall-wall."""
        self.bc_ymax_type = "dirichlet"
        self.bc_ymax_values = {"vel_v": 0.}
        self.bc_ymin_type = "dirichlet"
        self.bc_ymin_values = {"vel_v": 0.}

    def perio_xy(self):
        """Return x and y Periodicities booleans."""
        if "Perio" in [self.bc_xmax_type, self.bc_xmin_type]:
            if self.bc_xmax_type != self.bc_xmax_type:
                raise RuntimeError("One mismatched perio in x-direction")

        if "Perio" in [self.bc_ymax_type, self.bc_ymin_type]:
            if self.bc_ymax_type != self.bc_ymin_type:
                raise RuntimeError("One mismatched perio in x-direction")
        periox = False
        perioy = False
        if self.bc_xmax_type == "Perio":
            periox = True
        if self.bc_ymax_type == "Perio":
            perioy = True

        return periox, perioy

    def coord_matrix(self):
        """Compute coordiantes matrix."""
        x_coor = np.ones(self.shape) * np.linspace(
            0, self.dims[1], self.shape[1])[np.newaxis, :]
        y_coor = np.ones(self.shape) * np.linspace(
            0, self.dims[0], self.shape[0])[:, np.newaxis]
        return x_coor, y_coor

    def add_obstacle_circle(self, x_c=None, y_c=None, radius=None):
        """Add cicular obstacle.

        x_c = x center in m
        y_c = y center in m
        radius = radius in meters
        """
        if x_c is None:
            x_c = 0.3 * self.dims[1]
        if y_c is None:
            y_c = 0.5 * self.dims[0]
        if radius is None:
            radius = 0.1 * self.dims[0]
        x_coor, y_coor = self.coord_matrix()
        radius_field = np.hypot(x_coor - x_c, y_coor - y_c)
        self.obstacle = np.where(radius_field < radius, 1., self.obstacle)

    def add_obstacle_rect(self, ll_x=None, ll_y=None, ur_x=None, ur_y=None):
        """Add rectangular obstacle.

        ll_x = lower left x
        ll_y = lower left y
        ll_x = upper right x
        ll_y = upper right y
        """
        if ll_x is None:
            ll_x = 0.2 * self.dims[1]
        if ll_y is None:
            ll_y = 0.4 * self.dims[0]
        if ur_x is None:
            ur_x = 0.4 * self.dims[1]
        if ur_y is None:
            ur_y = 0.6 * self.dims[0]

        x_coor, y_coor = self.coord_matrix()
        rect_field = np.ones(self.shape)

        rect_field = np.where(x_coor < ll_x, 0., rect_field)
        rect_field = np.where(x_coor > ur_x, 0., rect_field)
        rect_field = np.where(y_coor < ll_y, 0., rect_field)
        rect_field = np.where(y_coor > ur_y, 0., rect_field)

        self.obstacle = np.maximum(rect_field, self.obstacle)

    def nullify_on_obstacle(self, field):
        """Nullify a field where the obstacle is defined."""
        return np.where(self.obstacle > 0., 0., field)

    def show_fields(self):
        """Show the 4 fields in a 4 graph panel."""
        x_linp1 = np.linspace(0, self.dims[1], self.shape[1]+1)
        y_linp1 = np.linspace(0, self.dims[0], self.shape[0]+1)
        max_vel = max(
            self.fields["vel_u"].max(),
            self.fields["vel_v"].max(),
            -self.fields["vel_u"].min(),
            -self.fields["vel_v"].min(), )

        fig = plt.figure()
        ax1 = fig.add_subplot(221)
        ax1.set_title('velu')
        plt.pcolormesh(
            x_linp1,
            y_linp1,
            self.fields["vel_u"],
            vmin=-max_vel,
            vmax=max_vel,
            cmap="bwr")
        plt.colorbar()
        ax1.set_aspect('equal')
        ax2 = fig.add_subplot(222)
        ax2.set_title('velv')
        ax2.set_aspect('equal')
        plt.pcolormesh(
            x_linp1,
            y_linp1,
            self.fields["vel_v"],
            vmin=-max_vel,
            vmax=max_vel,
            cmap="bwr")
        ax3 = fig.add_subplot(223)
        ax3.set_title('press')
        ax3.set_aspect('equal')
        plt.pcolormesh(
            x_linp1,
            y_linp1,
            self.fields["press"],
            cmap="Greys")
        plt.colorbar()
        ax4 = fig.add_subplot(224)
        ax4.set_title('scal')
        ax4.set_aspect('equal')
        plt.pcolormesh(
            x_linp1,
            y_linp1,
            self.fields["scal"],
            cmap="YlGn")
        plt.colorbar()
        plt.show()

    def show_flow(self):
        """Show the flow with magnitude and streamlines."""
        magv = np.hypot(self.fields["vel_u"], self.fields["vel_v"])

        x_lin = np.linspace(0, self.dims[1], self.shape[1])
        y_lin = np.linspace(0, self.dims[0], self.shape[0])

        x_linp1 = np.linspace(0, self.dims[1], self.shape[1]+1)
        y_linp1 = np.linspace(0, self.dims[0], self.shape[0]+1)

        plt.pcolormesh(x_linp1, y_linp1, magv, cmap="jet")
        plt.colorbar()
        plt.streamplot(
            x_lin,
            y_lin,
            self.fields["vel_u"],
            self.fields["vel_v"],
            density=[0.5, 1])
        plt.show()


def blur(arr_):
    """Apply a gather scater smothing to an array.

    Probably something smarted already exists in numpy/scipy.
    If someone find ...
    """
    kernel = np.array(
        [[1.0, 2.0, 1.0],
         [2.0, 4.0, 2.0],
         [1.0, 2.0, 1.0]])
    kernel = kernel / np.sum(kernel)
    arraylist = []
    for y_i in range(3):
        temparray = np.copy(arr_)
        temparray = np.roll(temparray, y_i - 1, axis=0)
        for x_i in range(3):
            temparray_x = np.copy(temparray)
            temparray_x = np.roll(temparray_x, x_i - 1, axis=1)*kernel[y_i, x_i]
            arraylist.append(temparray_x)
    arraylist = np.array(arraylist)
    arraylist_sum = np.sum(arraylist, axis=0)
    return arraylist_sum



"""Example on how to solve a pollution problem.

This example uses the navier stokes solver,
add a recording structure,
and control the whole test with a dictionnary.

This example is used by Smurf for data assimilation training.
See I. Mirouze for further details.
"""

import numpy as np


from barbatruc.fluid_domain import DomainRectFluid, blur
from barbatruc.fd_ns_2d import ns_iteration

__all__ = ["smurf_test"]


def smurf_test(dof, nsave):
    """Startup computation
    solve a pollution problem with the navier stokes solver
    """
    dimx = dof["nx"] * dof["dx"]
    dimy = dof["ny"] * dof["dx"]

    dom = DomainRectFluid(
        dimx=dimx,
        dimy=dimy,
        delta_x=dof["dx"],
        rho=dof["rho"],
        nu_=dof["nu"])

    # Add an initial speed
    dom.fields["vel_u"] = np.ones(dom.shape)*dof["u_west"]
    # Add velocity BC on the left of the domain
    dom.bcx_inlet_uv_outlet_p(val_u=dof["u_west"], val_v=0.0)

    # Add a source term on scalar.
    source = np.zeros(dom.shape)
    source[dof["source"]["location"]] = dof["source"]["value"]
    for _ in range(dof["source"]["blur_it"]):
        source = blur(source)
    dom.source["scal"] = source

    # Add monitoring array
    rec_field = dict()
    rec_shape = (nsave+1, dom.shape[0], dom.shape[1])
    for field in ["vel_u", "vel_v", "scal", "press"]:
        rec_field[field] = np.zeros(rec_shape)
    time_rec = np.zeros((nsave+1))

    # record inital state
    time_rec[0] = 0
    for field in ["vel_u", "vel_v", "scal", "press"]:
        rec_field[field][0, :, :] = dom.fields[field]

    # MAIN TEMPORAL LOOP--------------------
    t_end = dof["tend"]/1000
    time = 0.0
    time_step = t_end/nsave
    for i in range(nsave):

        time += time_step
        ns_iteration(dom, time_step)
        for field in ["vel_u", "vel_v", "scal", "press"]:
            rec_field[field][i+1, :, :] = dom.fields[field]
        time_rec[i+1] = time

        print('  Max u:', np.max(dom.fields["vel_u"]))
        print('  Time :', time)
        print('  Iteration %d' % (i))

    print('Normal end of execution.')
    return rec_field, time_rec

if __name__ == "__main__":

    DOF = {
        "rho": 1.0,
        "nu": 1.0,
        "nx": 40,
        "ny": 20,
        "dx": 0.02,
        "tend": 100.,
        "u_west": 20.,
        "source": {
            "location": (10, 10),
            "value": 1.0,
            "blur_it": 0
        }

    }

    REC, TIME = smurf_test(DOF, 20)
#    plt.matshow(REC["scal"][-1, :, :])
#    plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

REQUIRES = [
    "numpy",
    "matplotlib",
]
with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='barbatruc',
    version='0.0.2',
    description='Training computational fluid dynamics solver', 
    long_description=readme,
    author='CERFACS-COOP',
    author_email='coop@cerfacs.fr',
    url='https://nitrox.cerfacs.fr/open-source/barbatruc',
    license=license,
    packages=find_packages(exclude=('tests', 'docs')),
    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-cov'],
    install_requires=REQUIRES,
)

